@@ -1,6 +1,6 @@
-/* psync.c - Last modified: 03-Jul-2019 (kobayasy)
+/* psync.c - Last modified: 27-Feb-2020 (kobayasy)
  *
- * Copyright (c) 2018-2019 by Yuichi Kobayashi <kobayasy@kobayasy.com>
+ * Copyright (c) 2018-2020 by Yuichi Kobayashi <kobayasy@kobayasy.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -57,7 +57,7 @@
 #define LOADBUFFER_SIZE SYMLINK_MAX
 #endif  /* #if LOADBUFFER_SIZE < SYMLINK_MAX */
 
-ssize_t write_sp(int fd, const void *buf, size_t count) {
+ssize_t write_psync(int fd, const void *buf, size_t count) {
     ssize_t status = -1;
     size_t size;
     struct pollfd fds;
@@ -83,7 +83,8 @@ ssize_t write_sp(int fd, const void *buf, size_t count) {
 error:
     return status;
 }
-ssize_t read_sp(int fd, void *buf, size_t count) {
+
+ssize_t read_psync(int fd, void *buf, size_t count) {
     ssize_t status = -1;
     size_t size;
     struct pollfd fds;
@@ -110,7 +111,7 @@ error:
     return status;
 }
 
-int strcmp_sp(const char *s1, const char *s2) {
+int strcmp_psync(const char *s1, const char *s2) {
     int n = strcmp(s1, s2);
 
     if (n < 0) {
@@ -124,261 +125,6 @@ int strcmp_sp(const char *s1, const char *s2) {
     return n;
 }
 
-#define FST_UPLD  0x08
-#define FST_DNLD  0x80
-#define FST_LTYPE 0x07
-#define  FST_LREG 0x01
-#define  FST_LDIR 0x02
-#define  FST_LLNK 0x03
-#define FST_RTYPE 0x70
-#define  FST_RREG 0x10
-#define  FST_RDIR 0x20
-#define  FST_RLNK 0x30
-typedef struct {
-    time_t revision;
-    time_t mtime;
-    mode_t mode;
-    off_t size;
-    uint8_t flags;
-} FST;
-typedef struct s_flist {
-    struct s_flist *next, *prev;
-    FST st;
-    char name[1];
-} FLIST;
-typedef enum {
-    FSETS_1AND2,
-    FSETS_1NOT2,
-    FSETS_2NOT1
-} FSETS;
-
-static FLIST *new_f(FLIST *flist) {
-    LIST_NEW(flist);
-    memset(&flist->st, 0, sizeof(flist->st));
-    return flist;
-}
-
-static FLIST *add_f(FLIST *flist, const char *name) {
-    FLIST *fnew = NULL;
-
-    if (!*name)
-        goto error;
-    fnew = malloc(offsetof(FLIST, name) + strlen(name) + 1);
-    if (fnew == NULL)
-        goto error;
-    strcpy(fnew->name, name);
-    memset(&fnew->st, 0, sizeof(fnew->st));
-    LIST_INSERT_NEXT(fnew, flist);
-error:
-    return fnew;
-}
-
-static int each_next_f(FLIST *flist,
-                       int (*func)(FLIST *f, void *data),
-                                             void *data,
-                       volatile sig_atomic_t *stop ) {
-    int status = INT_MIN;
-    FLIST *f = flist;
-    FLIST *fnext;
-
-    if (*f->name)
-        goto error;
-    f = flist->next;
-    while (*f->name) {
-        ONSTOP(stop, -1);
-        fnext = f->next;
-        status = func(f, data);
-        if (ISERR(status))
-            goto error;
-        f = fnext;
-    }
-    status = 0;
-error:
-    return status;
-}
-
-#if 0  // not used
-static int each_prev_f(FLIST *flist,
-                       int (*func)(FLIST *f, void *data),
-                                             void *data,
-                       volatile sig_atomic_t *stop ) {
-    int status = INT_MIN;
-    FLIST *f = flist;
-    FLIST *fprev;
-
-    if (*f->name)
-        goto error;
-    f = flist->prev;
-    while (*f->name) {
-        ONSTOP(stop, -1);
-        fprev = f->prev;
-        status = func(f, data);
-        if (ISERR(status))
-            goto error;
-        f = fprev;
-    }
-    status = 0;
-error:
-    return status;
-}
-#endif  /* #if 0  // not used */
-
-static int each_sets_f(FLIST *flist1, FLIST *flist2,
-                       int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data),
-                                                                     void *data,
-                       volatile sig_atomic_t *stop ) {
-    int status = INT_MIN;
-    FLIST *f1 = flist1;
-    FLIST *f2 = flist2;
-    FLIST *f1next, *f2next;
-    int n;
-
-    if (*f1->name || *f2->name)
-        goto error;
-    f1 = f1->next, f2 = f2->next;
-    while (*f1->name || *f2->name) {
-        ONSTOP(stop, -1);
-        n = strcmp_sp(f1->name, f2->name);
-        if (n < 0) {
-            f1next = f1->next;
-            status = func(FSETS_1NOT2, f1, f2, data);
-            if (ISERR(status))
-                goto error;
-            f1 = f1next;
-        }
-        else if (n > 0) {
-            f2next = f2->next;
-            status = func(FSETS_2NOT1, f1, f2, data);
-            if (ISERR(status))
-                goto error;
-            f2 = f2next;
-        }
-        else {
-            f1next = f1->next, f2next = f2->next;
-            status = func(FSETS_1AND2, f1, f2, data);
-            if (ISERR(status))
-                goto error;
-            f1 = f1next, f2 = f2next;
-        }
-    }
-    status = 0;
-error:
-    return status;
-}
-
-static int write_f(bool synced, FLIST *flist, int fd,
-                   volatile sig_atomic_t *stop ) {
-    int status = INT_MIN;
-    size_t length, n;
-    FST st;
-
-    if (*flist->name) {
-        status = -1;
-        goto error;
-    }
-    for (flist = flist->next; *flist->name; flist = flist->next) {
-        ONSTOP(stop, -1);
-        n = length = strlen(flist->name);
-        WRITE_ONERR(n, fd, -1);
-        if (write_sp(fd, flist->name, length) != length) {
-            status = -1;
-            goto error;
-        }
-        st = flist->st;
-        WRITE_ONERR(st.revision, fd, -1);
-        WRITE_ONERR(st.mtime, fd, -1);
-        WRITE_ONERR(st.mode, fd, -1);
-        if (synced) {
-            WRITE_ONERR(st.size, fd, -1);
-            st.flags = st.flags >> 4 | st.flags << 4;
-            WRITE_ONERR(st.flags, fd, -1);
-        }
-    }
-    length = 0;
-    WRITE_ONERR(length, fd, -1);
-    status = 0;
-error:
-    return status;
-}
-
-static int read_f(bool synced, FLIST *flist, int fd,
-                  volatile sig_atomic_t *stop ) {
-    int status = INT_MIN;
-    size_t length;
-    char dirname[PATH_MAX];
-
-    if (*flist->name) {
-        status = -1;
-        goto error;
-    }
-    READ_ONERR(length, fd, -1);
-    while (length > 0) {
-        ONSTOP(stop, -1);
-        if (length > sizeof(dirname)-1) {
-            status = -1;
-            goto error;
-        }
-        if (read_sp(fd, dirname, length) != length) {
-            status = -1;
-            goto error;
-        }
-        dirname[length] = 0;
-        flist = add_f(flist, dirname);
-        if (flist == NULL) {
-            status = -1;
-            goto error;
-        }
-        READ_ONERR(flist->st.revision, fd, -1);
-        READ_ONERR(flist->st.mtime, fd, -1);
-        READ_ONERR(flist->st.mode, fd, -1);
-        if (synced) {
-            READ_ONERR(flist->st.size, fd, -1);
-            READ_ONERR(flist->st.flags, fd, -1);
-        }
-        READ_ONERR(length, fd, -1);
-    }
-    status = 0;
-error:
-    return status;
-}
-
-static int delete_func(FLIST *f, void *data) {
-    int status = INT_MIN;
-
-    LIST_DELETE(f);
-    free(f);
-    status = 0;
-    return status;
-}
-
-#define FILEID 0x01665370  /* 'p', 'S', 'f', 1 */
-#define SYNCDIR  ".psync"
-#define LASTFILE "last"
-#define LOCKDIR  "lock"
-#define BACKDIR  "%Y%m%d%H%M.%S%z"
-#define UPFILE   "u%lu"
-#define DOWNFILE "d%lu"
-#define BACKFILE "%lu,%s"
-#define LOGFILE  "log"
-typedef struct {
-    time_t t;
-    time_t expire;
-    time_t backup;
-    int fdin, fdout;
-    int info;
-    volatile sig_atomic_t *stop;
-    time_t tlast;
-    FLIST fsynced;
-    FLIST flocal, fremote;
-    char dirname[1];
-} PRIV;
-
-#define INFO(_priv, ...) \
-    do { \
-        if ((_priv)->info != -1) \
-            dprintf((_priv)->info, __VA_ARGS__); \
-    } while (0)
-
 typedef struct {
     int fd;
     unsigned long interval;
@@ -443,6 +189,160 @@ static int progress_term(PROGRESS *progress) {
     return status;
 }
 
+#define FST_UPLD  0x08
+#define FST_DNLD  0x80
+#define FST_LTYPE 0x07
+#define  FST_LREG 0x01
+#define  FST_LDIR 0x02
+#define  FST_LLNK 0x03
+#define FST_RTYPE 0x70
+#define  FST_RREG 0x10
+#define  FST_RDIR 0x20
+#define  FST_RLNK 0x30
+typedef struct {
+    time_t revision;
+    time_t mtime;
+    mode_t mode;
+    off_t size;
+    uint8_t flags;
+} FST;
+typedef struct s_flist {
+    struct s_flist *next, *prev;
+    FST st;
+    char name[1];
+} FLIST;
+
+static FLIST *new_FLIST(FLIST *flist) {
+    LIST_NEW(flist);
+    memset(&flist->st, 0, sizeof(flist->st));
+    return flist;
+}
+
+static FLIST *add_FLIST(FLIST *flist, const char *name) {
+    FLIST *fnew = NULL;
+
+    if (!*name)
+        goto error;
+    fnew = malloc(offsetof(FLIST, name) + strlen(name) + 1);
+    if (fnew == NULL)
+        goto error;
+    strcpy(fnew->name, name);
+    memset(&fnew->st, 0, sizeof(fnew->st));
+    LIST_INSERT_NEXT(fnew, flist);
+error:
+    return fnew;
+}
+
+static each_next(FLIST)
+
+static sets_next(FLIST)
+
+static int write_FLIST(bool synced, FLIST *flist, int fd,
+                       volatile sig_atomic_t *stop ) {
+    int status = INT_MIN;
+    size_t length, n;
+    FST st;
+
+    if (*flist->name) {
+        status = -1;
+        goto error;
+    }
+    for (flist = flist->next; *flist->name; flist = flist->next) {
+        ONSTOP(stop, -1);
+        n = length = strlen(flist->name);
+        WRITE_ONERR(n, fd, -1);
+        if (write_psync(fd, flist->name, length) != length) {
+            status = -1;
+            goto error;
+        }
+        st = flist->st;
+        WRITE_ONERR(st.revision, fd, -1);
+        WRITE_ONERR(st.mtime, fd, -1);
+        WRITE_ONERR(st.mode, fd, -1);
+        if (synced) {
+            WRITE_ONERR(st.size, fd, -1);
+            st.flags = st.flags >> 4 | st.flags << 4;
+            WRITE_ONERR(st.flags, fd, -1);
+        }
+    }
+    length = 0;
+    WRITE_ONERR(length, fd, -1);
+    status = 0;
+error:
+    return status;
+}
+
+static int read_FLIST(bool synced, FLIST *flist, int fd,
+                      volatile sig_atomic_t *stop ) {
+    int status = INT_MIN;
+    size_t length;
+    char dirname[PATH_MAX];
+
+    if (*flist->name) {
+        status = -1;
+        goto error;
+    }
+    READ_ONERR(length, fd, -1);
+    while (length > 0) {
+        ONSTOP(stop, -1);
+        if (length > sizeof(dirname)-1) {
+            status = -1;
+            goto error;
+        }
+        if (read_psync(fd, dirname, length) != length) {
+            status = -1;
+            goto error;
+        }
+        dirname[length] = 0;
+        flist = add_FLIST(flist, dirname);
+        if (flist == NULL) {
+            status = -1;
+            goto error;
+        }
+        READ_ONERR(flist->st.revision, fd, -1);
+        READ_ONERR(flist->st.mtime, fd, -1);
+        READ_ONERR(flist->st.mode, fd, -1);
+        if (synced) {
+            READ_ONERR(flist->st.size, fd, -1);
+            READ_ONERR(flist->st.flags, fd, -1);
+        }
+        READ_ONERR(length, fd, -1);
+    }
+    status = 0;
+error:
+    return status;
+}
+
+static int delete_func(FLIST *f, void *data) {
+    int status = INT_MIN;
+
+    LIST_DELETE(f);
+    free(f);
+    status = 0;
+    return status;
+}
+
+#define SYNCDIR  ".psync"
+#define LASTFILE "last"
+#define LOCKDIR  "lock"
+#define BACKDIR  "%Y%m%d%H%M.%S%z"
+#define UPFILE   "u%lu"
+#define DOWNFILE "d%lu"
+#define BACKFILE "%lu,%s"
+#define LOGFILE  "log"
+typedef struct {
+    time_t t;
+    time_t expire;
+    time_t backup;
+    int fdin, fdout;
+    int info;
+    volatile sig_atomic_t *stop;
+    time_t tlast;
+    FLIST fsynced;
+    FLIST flocal, fremote;
+    char dirname[1];
+} PRIV;
+
 static int lock(PRIV *priv) {
     int status = INT_MIN;
     char loadname[PATH_MAX], *p;
@@ -488,9 +388,9 @@ static PRIV *new_priv(const char *dirname,
     priv->info = -1;
     priv->stop = stop;
     priv->tlast = -1;
-    new_f(&priv->fsynced);
-    new_f(&priv->flocal);
-    new_f(&priv->fremote);
+    new_FLIST(&priv->fsynced);
+    new_FLIST(&priv->flocal);
+    new_FLIST(&priv->fremote);
     if (lock(priv)) {
         free(priv), priv = NULL;
         goto error;
@@ -501,9 +401,9 @@ error:
 
 static void free_priv(PRIV *priv) {
     unlock(priv);
-    each_next_f(&priv->fsynced, delete_func, NULL, NULL);
-    each_next_f(&priv->flocal, delete_func, NULL, NULL);
-    each_next_f(&priv->fremote, delete_func, NULL, NULL);
+    each_next_FLIST(&priv->fsynced, delete_func, NULL, NULL);
+    each_next_FLIST(&priv->flocal, delete_func, NULL, NULL);
+    each_next_FLIST(&priv->fremote, delete_func, NULL, NULL);
     free(priv);
 }
 
@@ -520,9 +420,9 @@ static int save_fsynced(PRIV *priv) {
         status = ERROR_DMAKE;
         goto error;
     }
-    id = FILEID;
+    id = PSYNC_FILEID;
     WRITE_ONERR(id, fd, ERROR_DWRITE);
-    status = write_f(false, &priv->fsynced, fd, priv->stop);
+    status = write_FLIST(false, &priv->fsynced, fd, priv->stop);
     ONSTOP(priv->stop, ERROR_STOP);
     if (ISERR(status)) {
         status = ERROR_DWRITE;
@@ -561,8 +461,8 @@ static int load_fsynced(PRIV *priv) {
         goto error;
     }
     READ(id, fd, n);
-    if (!ISERR(n) && id == FILEID) {
-        status = read_f(false, &priv->fsynced, fd, priv->stop);
+    if (!ISERR(n) && id == PSYNC_FILEID) {
+        status = read_FLIST(false, &priv->fsynced, fd, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status)) {
             status = ERROR_DREAD;
@@ -619,7 +519,7 @@ static int get_flocal_r(FLIST **flocal, FLIST **flast, char *pathname, char *nam
     LIST_SEEK_NEXT(*flocal, name, seek);
     LIST_SEEK_NEXT(*flast, name, seek);
     if (seek) {
-        *flocal = add_f(*flocal, name);
+        *flocal = add_FLIST(*flocal, name);
         if (*flocal == NULL) {
             status = ERROR_MEMORY;
             goto error;
@@ -717,18 +617,18 @@ error:
     return status;
 }
 
-static int add_deleted_func(FSETS sets, FLIST *flocal, FLIST *flast, void *data) {
+static int add_deleted_func(SETS sets, FLIST *flocal, FLIST *flast, void *data) {
     int status = INT_MIN;
     PRIV *priv = data;
 
     switch (sets) {
-    case FSETS_1AND2:
+    case SETS_1AND2:
         LIST_DELETE(flast);
         free(flast);
         break;
-    case FSETS_1NOT2:
+    case SETS_1NOT2:
         break;
-    case FSETS_2NOT1:
+    case SETS_2NOT1:
         LIST_DELETE(flast);
         switch (flast->st.mode & S_IFMT) {
         case 0:  /* deleted */
@@ -750,12 +650,12 @@ static int add_deleted_func(FSETS sets, FLIST *flocal, FLIST *flast, void *data)
     return status;
 }
 
-static int make_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+static int make_fsynced_func(SETS sets, FLIST *flocal, FLIST *fremote, void *data) {
     int status = INT_MIN;
     PRIV *priv = data;
 
     switch (sets) {
-    case FSETS_1AND2:
+    case SETS_1AND2:
         if (flocal->st.revision < fremote->st.revision) {
             fremote->st.flags |= flocal->st.flags;
             if (flocal->st.mtime != fremote->st.mtime)
@@ -776,12 +676,12 @@ static int make_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *da
             LIST_INSERT_PREV(flocal, &priv->fsynced);
         }
         break;
-    case FSETS_1NOT2:
+    case SETS_1NOT2:
         flocal->st.flags |= FST_UPLD;
         LIST_DELETE(flocal);
         LIST_INSERT_PREV(flocal, &priv->fsynced);
         break;
-    case FSETS_2NOT1:
+    case SETS_2NOT1:
         fremote->st.flags |= FST_DNLD;
         LIST_DELETE(fremote);
         LIST_INSERT_PREV(fremote, &priv->fsynced);
@@ -791,12 +691,12 @@ static int make_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *da
     return status;
 }
 
-static int make_fsynced_put_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+static int make_fsynced_put_func(SETS sets, FLIST *flocal, FLIST *fremote, void *data) {
     int status = INT_MIN;
     PRIV *priv = data;
 
     switch (sets) {
-    case FSETS_1AND2:
+    case SETS_1AND2:
         flocal->st.flags |= fremote->st.flags;
         if (flocal->st.revision != fremote->st.revision &&
             flocal->st.mtime != fremote->st.mtime )
@@ -806,12 +706,12 @@ static int make_fsynced_put_func(FSETS sets, FLIST *flocal, FLIST *fremote, void
         LIST_DELETE(flocal);
         LIST_INSERT_PREV(flocal, &priv->fsynced);
         break;
-    case FSETS_1NOT2:
+    case SETS_1NOT2:
         flocal->st.flags |= FST_UPLD;
         LIST_DELETE(flocal);
         LIST_INSERT_PREV(flocal, &priv->fsynced);
         break;
-    case FSETS_2NOT1:
+    case SETS_2NOT1:
         LIST_DELETE(fremote);
         free(fremote);
         break;
@@ -820,20 +720,21 @@ static int make_fsynced_put_func(FSETS sets, FLIST *flocal, FLIST *fremote, void
     return status;
 }
 
-static int make_fsynced_get_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+static int make_fsynced_get_func(SETS sets, FLIST *flocal, FLIST *fremote, void *data) {
     return make_fsynced_put_func(sets, fremote, flocal, data);
 }
 
 static int preload(PRIV *priv) {
     int status = INT_MIN;
-    intmax_t uploadsize, downloadsize;
+    intmax_t uploadsize;
+    PROGRESS progress;
     char pathname[PATH_MAX], *name;
     char loadname[PATH_MAX], *p;
     unsigned long count;
     FLIST *fsynced;
     char buffer[SYMLINK_MAX];
 
-    uploadsize = 0, downloadsize = 0;
+    progress_init(&progress, priv->info, PROGRESS_INTERVAL, "U%+jd\n", uploadsize = 0);
     name = pathname, name += sprintf(name, "%s/", priv->dirname);
     p = loadname, p += sprintf(p, "%s/"SYNCDIR"/"LOCKDIR"/", priv->dirname);
     count = 0;
@@ -862,17 +763,11 @@ static int preload(PRIV *priv) {
                 }
                 break;
             }
-            uploadsize += fsynced->st.size;
-            break;
-        }
-        switch (fsynced->st.flags & (FST_DNLD|FST_RTYPE)) {
-        case FST_DNLD|FST_RREG:
-        case FST_DNLD|FST_RLNK:
-            downloadsize += fsynced->st.size;
+            progress_update(&progress, uploadsize += fsynced->st.size);
             break;
         }
     }
-    INFO(priv, "P%+jd%+jd\n", uploadsize, downloadsize);
+    progress_term(&progress);
     status = 0;
 error:
     return status;
@@ -880,8 +775,6 @@ error:
 
 static int upload(PRIV *priv) {
     int status = INT_MIN;
-    intmax_t uploadsize;
-    PROGRESS progress;
     char loadname[PATH_MAX], *p;
     unsigned long count;
     FLIST *fsynced;
@@ -890,7 +783,6 @@ static int upload(PRIV *priv) {
     ssize_t n;
     char buffer[LOADBUFFER_SIZE];
 
-    progress_init(&progress, priv->info, PROGRESS_INTERVAL, "U%+jd\n", uploadsize = 0);
     p = loadname, p += sprintf(p, "%s/"SYNCDIR"/"LOCKDIR"/", priv->dirname);
     count = 0;
     for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
@@ -909,17 +801,16 @@ static int upload(PRIV *priv) {
                 }
                 while (size > 0) {
                     ONSTOP(priv->stop, ERROR_STOP);
-                    n = read_sp(fd, buffer, size > sizeof(buffer) ? sizeof(buffer) : size);
+                    n = read_psync(fd, buffer, size > sizeof(buffer) ? sizeof(buffer) : size);
                     if (n == -1) {
                         status = ERROR_FREAD;
                         goto error;
                     }
-                    if (write_sp(priv->fdout, buffer, n) != n) {
+                    if (write_psync(priv->fdout, buffer, n) != n) {
                         status = ERROR_FUPLD;
                         goto error;
                     }
                     size -= n;
-                    progress_update(&progress, uploadsize += n);
                 }
                 close(fd), fd = -1;
                 break;
@@ -928,11 +819,10 @@ static int upload(PRIV *priv) {
                     status = ERROR_FREAD;
                     goto error;
                 }
-                if (write_sp(priv->fdout, buffer, size) != size) {
+                if (write_psync(priv->fdout, buffer, size) != size) {
                     status = ERROR_FUPLD;
                     goto error;
                 }
-                progress_update(&progress, uploadsize += size);
                 break;
             }
             if (unlink(loadname) == -1) {
@@ -942,7 +832,6 @@ static int upload(PRIV *priv) {
             break;
         }
     }
-    progress_term(&progress);
     status = 0;
 error:
     if (fd != -1)
@@ -982,12 +871,12 @@ static int download(PRIV *priv) {
                 }
                 while (size > 0) {
                     ONSTOP(priv->stop, ERROR_STOP);
-                    n = read_sp(priv->fdin, buffer, size > sizeof(buffer) ? sizeof(buffer) : size);
+                    n = read_psync(priv->fdin, buffer, size > sizeof(buffer) ? sizeof(buffer) : size);
                     if (n == -1) {
                         status = ERROR_FDNLD;
                         goto error;
                     }
-                    if (write_sp(fd, buffer, n) != n) {
+                    if (write_psync(fd, buffer, n) != n) {
                         status = ERROR_FWRITE;
                         goto error;
                     }
@@ -1005,7 +894,7 @@ static int download(PRIV *priv) {
                     status = ERROR_SYSTEM;
                     goto error;
                 }
-                if (read_sp(priv->fdin, buffer, size) != size) {
+                if (read_psync(priv->fdin, buffer, size) != size) {
                     status = ERROR_FDNLD;
                     goto error;
                 }
@@ -1185,7 +1074,7 @@ static int make_log(PRIV *priv) {
             break;
         }
     }
-    fprintf(fp, "%lu deleted, %lu added, %lu modified, %lu uploaded\n",
+    fprintf(fp, "%lu deleted, %lu added, %lu modified / %lu uploaded\n",
             count.deleted, count.added, count.modified, count.uploaded );
     for (fsynced = priv->fsynced.next; *fsynced->name; fsynced = fsynced->next) {
         ONSTOP(priv->stop, ERROR_STOP);
@@ -1364,13 +1253,15 @@ typedef struct {
     int status;
     pthread_t tid;
 } LOAD_PARAM;
+
 static void *upload_thread(void *data) {
     LOAD_PARAM *param = data;
 
     param->status = upload(param->priv);
     return NULL;
 }
-static int run(int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *priv) {
+
+static int run(int (*func)(SETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *priv) {
     int status = INT_MIN;
     LOAD_PARAM upload_param = {
         .priv   = priv,
@@ -1384,22 +1275,22 @@ static int run(int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *
     if (ISERR(status))
         goto error;
     ONSTOP(priv->stop, ERROR_STOP);
-    status = each_sets_f(&priv->flocal, &priv->fsynced, add_deleted_func, priv, priv->stop);
+    status = sets_next_FLIST(&priv->flocal, &priv->fsynced, add_deleted_func, priv, priv->stop);
     ONSTOP(priv->stop, ERROR_STOP);
     if (ISERR(status))
         goto error;
     if (func != NULL) {
-        status = read_f(true, &priv->fremote, priv->fdin, priv->stop);
+        status = read_FLIST(true, &priv->fremote, priv->fdin, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status)) {
             status = ERROR_SDNLD;
             goto error;
         }
-        status = each_sets_f(&priv->flocal, &priv->fremote, func, priv, priv->stop);
+        status = sets_next_FLIST(&priv->flocal, &priv->fremote, func, priv, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status))
             goto error;
-        status = write_f(true, &priv->fsynced, priv->fdout, priv->stop);
+        status = write_FLIST(true, &priv->fsynced, priv->fdout, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status)) {
             status = ERROR_SUPLD;
@@ -1407,17 +1298,17 @@ static int run(int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *
         }
     }
     else {
-        status = write_f(true, &priv->flocal, priv->fdout, priv->stop);
+        status = write_FLIST(true, &priv->flocal, priv->fdout, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status)) {
             status = ERROR_SUPLD;
             goto error;
         }
-        status = each_next_f(&priv->flocal, delete_func, NULL, priv->stop);
+        status = each_next_FLIST(&priv->flocal, delete_func, NULL, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status))
             goto error;
-        status = read_f(true, &priv->fsynced, priv->fdin, priv->stop);
+        status = read_FLIST(true, &priv->fsynced, priv->fdin, priv->stop);
         ONSTOP(priv->stop, ERROR_STOP);
         if (ISERR(status)) {
             status = ERROR_SDNLD;
@@ -1475,7 +1366,7 @@ void psync_free(PSYNC *psync) {
 }
 
 int psync_run(PSYNC_MODE mode, PSYNC *psync) {
-    int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data);
+    int (*func)(SETS sets, FLIST *f1, FLIST *f2, void *data);
 
     switch (mode) {
     case PSYNC_MASTER:
